@@ -736,7 +736,7 @@ static int ixp4xx_npe_probe(struct platform_device *pdev)
 	return 0;
 }
 
-static int ixp4xx_npe_remove(struct platform_device *pdev)
+static void ixp4xx_npe_remove(struct platform_device *pdev)
 {
 	int i;
 
@@ -744,8 +744,6 @@ static int ixp4xx_npe_remove(struct platform_device *pdev)
 		if (npe_tab[i].regs) {
 			npe_reset(&npe_tab[i]);
 		}
-
-	return 0;
 }
 
 static const struct of_device_id ixp4xx_npe_of_match[] = {
@@ -766,6 +764,7 @@ static struct platform_driver ixp4xx_npe_driver = {
 module_platform_driver(ixp4xx_npe_driver);
 
 MODULE_AUTHOR("Krzysztof Halasa");
+MODULE_DESCRIPTION("Intel IXP4xx Network Processor Engine driver");
 MODULE_LICENSE("GPL v2");
 /*(DEBLOBBED)*/
 
